
/*---------------------------------------------------------------------------*/

/*
**  Data types.  Should work for both 16-bit and 32-bit systems.
*/

typedef unsigned char  BytT, *BytP;     /* 8-bit unsigned value  */
typedef          char  ChrT, *ChrP;     /* 8-bit signed value    */
typedef unsigned short WrdT, *WrdP;     /* 16-bit unsigned value */
typedef   signed short IntT, *IntP;     /* 16-bit signed value   */
typedef unsigned long  DwdT, *DwdP;     /* 32-bit unsigned value */
typedef   signed long  LngT, *LngP;     /* 32-bit signed value   */

#define FALSE   0
#define TRUE    !FALSE


/*---------------------------------------------------------------------------*/

/*
**  # name = .PRM file structure and definitions
*/

#define OFS             WrdT
#define MAX_DRIVES        26    /* Maximum number of drives on system   */
#define CHAR_BITS          8    /* Number of bits in a 'char'           */
#define CTL_VER            9    /* Version number of BBS.PRM            */
#define MAX_LANG           8    /* Max. number of possible languages    */
#define MAX_YELL          10    /* Max number of yell slots             */
#define MAX_EXTERNP       16    /* max. number of external programs     */
#define ALIAS_CNT         15    /* number of matrix addresses           */
#define MAXCLASS          12    /* OBSOLETE!  Only used for Max 2.x     */

#define PRM(s) (offsets+(maxprm.s))

struct   cl_rec
{
  IntT     priv;
  WrdT     max_time;      /* max cume time per day         */
  WrdT     max_call;      /* max time for one call         */
  WrdT     max_dl;        /* max dl kbytes per day         */
  WrdT     ratio;         /* ul:dl ratio                   */
  WrdT     min_baud;      /* speed needed for logon        */
  WrdT     min_file_baud; /* speed needed for file xfer    */
};

typedef struct
{
    WrdT zone;
    WrdT net;
    WrdT node;
    WrdT point;
}
NETADDR;

struct m_pointers
{
  BytT  id;             /* Always equal to 'M'               STABLE  */
  BytT  version;        /* for safety                        STABLE  */
  WrdT  heap_offset;    /* OFFSET OF BEGINNING OF HEAP!      STABLE  */
  BytT  task_num;       /* for multi-tasking systems         STABLE  */
  IntT  com_port;       /* Com1=0, Com2=1, etc               STABLE  */
  BytT  noise_ok;       /* If yell noise is currently on     STABLE  */

  /* Miscellanious system information */

  BytT  video;          /* Mode for local video display              */
  BytT  log_mode;       /* What style of logging to use              */
  WrdT  max_baud;       /* fastest speed we can use                  */
  BytT  multitasker;    /* flag for DoubleDos (see below)            */
  BytT  nlver;          /* Which nodelist version we use (NLVER_XXX) */

  IntT  min_ulist;      /* OBSOLETE!  Only used for compatibility    */
  IntT  max_ulist;      /* with Maximus 2.x!                         */

  /* Information about errorlevels */

  BytT  exit_val;       /* Erl to use after caller if none of below  */
  BytT  edit_exit;      /* erl to use after matrix mail written      */
  BytT  echo_exit;      /* ERRORLEVEL for after inbound echomail     */
  BytT  local_exit;     /* Errorlevel after entering local msgs      */

  /* Modem information */

  IntT  carrier_mask;
  IntT  handshake_mask;

  /* Log-on information */

  WrdT  logon_priv;     /* Access level for new users                */
  WrdT  logon_time;     /* time to give for logons                   */
  WrdT  min_baud;       /* minimum baud to get on-line               */
  WrdT  speed_graphics; /* min baud for graphics                  */

  /* Information about message areas */

  BytT  auto_kill;      /* RECD PVT msgs. 0=no 1=ask 2=yes            */

  WrdT ctla_priv;       /* Priv to see CONTROL-A lines in messages    */
  WrdT seenby_priv;     /* Min priv to see SEEN-BY line               */
  WrdT pvt_priv;        /* Min priv to read pvt msgs                  */

  WrdT msg_ask[16];     /* Array of privs. for message attr ask's     */
  WrdT msg_assume[16];  /* Array of privs. for message attr assume's  */
  WrdT msg_fromfile;    /* Priv. for doing message from file          */
  WrdT speed_rip;       /* min baud for rip graphics                  */
  BytT rsvd1[2];        /* used to be high_msgarea, begin_msgarea     */
  WrdT unlisted_priv;   /* Priv needed to send to unlisted node       */
  IntT unlisted_cost;   /* Charge to send to unlisted node            */

  WrdT mc_reply_priv;   /* Priv to reply to msg with mailchecker     */
  WrdT mc_kill_priv;    /* Priv to kill msg with mailchecker         */


  /* Information about file areas */

  IntT  date_style;     /* Used for FILES.BBS display                */
  WrdT rsvd20;          /* Reserved (used to be dlall_priv)          */
  WrdT rsvd21;          /* Reserved (used to be ulbbs_priv)          */
  DwdT k_free;          /* The number of disk space (in K) which     *
                         * must be available before we let a user    *
                         * upload.                                   */
  WrdT  rsvd7;          /* reserved - used to be upload reward       */
  WrdT  ratio_threshold;/* OBSOLETE!  Only used for Max 2.x compat.  */
  BytT  rsvd2[4];       /* used to be high_filearea, begin_filearea  */

  /* Our matrix address(es) */

  NETADDR address[ALIAS_CNT];


  /*  struct _yell yell[MAX_YELL]; */  /* Yell info moved to event file */
  BytT rsvd3[60]; /* Reserved by Maximus for future use */


  /* About the users */
  struct cl_rec cls[MAXCLASS];  /* OBSOLETE!  Only used for compatibility */
                                /* with Maximus 2.x.                      */

  /* Flags for external protocols */

  IntT  protoexit;              /* Errorlevel for protocol exit      */
  char  protoflag[MAX_EXTERNP]; 

  /* General-purpose bit-flags  (See FLAGx_xxx definitions above.) */

  WrdT  flags;
  WrdT  flags2;
  WrdT  flags3;
  WrdT  flags4;

  /* Bit field containing drive letters to save when going outside */
  char  drives_to_save[(MAX_DRIVES/CHAR_BITS)+1];

  BytT  fbbs_margin;      /* Margin to use for wrapping FILES.BBS comments */
  BytT  kill_attach;      /* Kill file attaches 0=no 1=ask 2=yes           */

  WrdT  max_ptrs;         /* Maximum size of pointers of ALL *.LTFs */
  WrdT  max_heap;         /* Maximus heap size of all *.LTFs */
  BytT  max_lang;         /* Current number of languages */
  BytT  rsvd_lang;

  WrdT  max_glh_ptrs;
  WrdT  max_glh_len;

  WrdT  max_syh_ptrs;
  WrdT  max_syh_len;

  BytT  input_timeout;   /* # of mins until Max hangs up due to no input    */

  BytT  charset;         /* Character set support - see CHARSET_XXXX, above */
  WrdT  max_pack;        /* Maximum # of msgs to pack into a .QWK packet    */
  WrdT  msg_localattach; /* Priv. for doing local file attaches             */
  WrdT  kill_attach_priv;/* Priv. for killing local file attaches           */
  WrdT  mcp_sessions;    /* Max number of MCP sessions                      */
  DwdT  max_msgsize;     /* Truncate uploaded messages to this size         */
  
  BytT  rsvd65[2];       /* Reserved by Maximus for future use              */



  /* --------------------------------------------------------------- */
  /* -------------------------- OFFSETS ---------------------------- */
  /* --------------------------------------------------------------- */

  /* About your system */

  OFS   sysop;          /* sysop's name. MUST be first offset in prm file */
  OFS   system_name;    /* board's name                              */

  /* Modem commands */

  OFS   m_busy;         /* mdm cmd to take modem off hook            */

  /* Paths to various places */

  OFS   sys_path;       /* path to SYSTEM?.BBS files                 */
  OFS   misc_path;      /* path to `F-key files'                     */
  OFS   net_info;       /* path to NODELIST files                    */
  OFS   temppath;       /* place to put temporary files              */
  OFS   ipc_path;       /* path for inter-process communications     */

  /* General files needed by the system */

  OFS   user_file;      /* path/filename of User.Bbs                 */
  OFS   log_name;       /* name of the log file                      */
  OFS   chat_prog;      /* External chat program, if any             */
  OFS   chat_fbegin;    /* File to display instead of "CHAT: begin"  */
  OFS   chat_fend;      /* File to display instead of "END CHAT"     */
  OFS   local_editor;   /* Command for local editor, if any          */
  OFS   notfound;       /* User name not found in user file          */
  OFS   junk;           /* Don't use this for anything!              */

  /* General *.?BS files needed everywhere */

  OFS   logo;           /* first file shown to a caller              */
  OFS   bad_logon;      /* if user's last logon attempt was bad      */
  OFS   welcome;        /* shown after logon                         */
  OFS   quote;          /* For displaying "random" quotes from       */
  OFS   newuser1;       /* Asks user to type in password             */
  OFS   newuser2;       /* Replaces `welcome' for a new user         */
  OFS   rookie;         /* Replaces `welcome' for rookies            */
  OFS   application;    /* new user questionnaire                    */
  OFS   byebye;         /* file displayed at logoff                  */
  OFS   out_leaving;    /* Bon Voyage                                */
  OFS   out_return;     /* Welcome back from O)utside                */
  OFS   daylimit;       /* Sorry, you've been on too long...         */
  OFS   timewarn;       /* warning about forced hangup               */
  OFS   tooslow;        /* explains minimum logon baud rate          */
  OFS   barricade;      /* Displayed before prompt for access code   */
  OFS   shelltodos;     /* Displayed when Sysop hits Alt-J           */
  OFS   backfromdos;    /* Displayed when Sysop returns from Alt-J   */
  OFS   areanotexist;   /* File to display instead of "That area     *
                         * doesn't exist!"                           */

  /* User priv levels database */

  OFS   access;         /* Database of user privilege levels         */

  /* File-area items */

  OFS   xferbaud;       /* explains minimum file transfer baud rate  */
  OFS   file_area_list; /* dump file... used instead of Dir.Bbs      */
  OFS   no_space;       /* File to display if trying to UL with      *
                         * less than k_free space left on drive.     */
  OFS   fname_format;   /* Essay on MS-DOS filenames for U)ploads    */
  OFS   ul_log;         /* Log file for uploads                      */

  OFS   file_header;    /* Format for file area's A)rea command      */
  OFS   file_format;    /* Format for A)rea command entries          */
  OFS   file_footer;    /* Format for footer for file.area menu      */

  OFS   proto_dump;      /* Dump file for protocol screen            */

  /* Message-area items */

  OFS   msgarea_list;   /* dump file... used instead of Dir.Bbs      */
  OFS   echotoss_name;  /* Name of your echomail tosslog             */
  OFS   nomail;         /* Display by mailchecker if no mail wtng.   */

  OFS   msg_header;     /* Format for msg.area's A)rea command       */
  OFS   msg_format;     /* Format for A)reas command entries         */
  OFS   msg_footer;     /* Format for footer for msg.area menu       */

  /* Help files:  Used to explain various things */

  OFS   hlp_editor;     /* intro to msg editor for novices.          */
  OFS   hlp_replace;    /* Explain the Msg.Editor E)dit command      */
  OFS   msg_inquire;    /* Explain the Msg. I)nquire command         */
  OFS   hlp_locate;     /* Explain the Files L)ocate command         */
  OFS   hlp_contents;   /* Explain the Files C)ontents command       */
  OFS   oped_help;      /* help file for the full-screen editor      */
  OFS   hlp_scan;       /* help file for S)can                       */
  OFS   hlp_list;       /* help file for L)ist                       */

  /* External protocols */

  OFS   protocols[MAX_EXTERNP]; /* external file protocol programs   */
  OFS   protoname[MAX_EXTERNP]; /* name of protocol, on menu         */

  /* Date/Time format strings */

  OFS   timeformat;
  OFS   dateformat;

  /* **OBSOLETE** names of the old Max 2.x area.dat/idx files */

  OFS   adat_name;
  OFS   aidx_name;

  /* Menu paths/names */

  OFS   menupath;        /* The default place to look for the menus */
  OFS   first_menu;      /* The name of the first menu to display */
  OFS   edit_menu;       /* Name of the EDIT menu */
  
  /* Miscellaneous */
  
  OFS   achg_keys;       /* Characters used to change area -/+ */
  OFS   tune_file;       /* Path to TUNES.MAX */
  OFS   lang_path;       /* Path to *.LTF files */
  
  OFS   lang_file[MAX_LANG]; /* Array of all *.LTF names */
  
  OFS   m_init;          /* Modem initialization string */
  OFS   m_ring;          /* Command modem sends when phone ringing */
  OFS   m_answer;        /* Cmd to send to modem when ring detect */
  OFS   m_connect;       /* Connect string, as returned by modem */

  OFS   high_msgarea;
  OFS   begin_msgarea;  /* Msg area to put new users in              */
  
  OFS   high_filearea;
  OFS   begin_filearea; /* File area to put new users in             */
  
  OFS   fidouser;       /* Name of FIDOUSER.LST file to use          */
  OFS   cmtarea;        /* Message area to put comments in           */

  OFS   arc_ctl;        /* Control file for archiving programs       */
  OFS   olr_name;       /* OLR: Filename to use for DL packets       */
  OFS   olr_dir;        /* OLR: Directory for off-line stuff         */
  OFS   phone_num;
  OFS   viruschk;       /* Name of batch file to call for virus check*/
  OFS   protocol_max;   /* Name of compiled protocol data file       */
  OFS   track_privview; /* Priv req'd to see tracking info owned by others */
  OFS   track_privmod;  /* Priv req'd to modify track info owned by others */
  OFS   track_base;     /* Base name of the tracking database files  */
  OFS   track_exclude;  /* File containing list of names to exclude  */
  OFS   stagepath;      /* Path for staged CD-ROM transfers          */
  OFS   attach_base;    /* Base name of the local attach database    */
  OFS   attach_path;    /* Default path for local file attaches      */
  OFS   attach_archiver;/* File attach archiver                      */
  OFS   inbound_files;  /* Inbound FTS-1 attaches for users          */
  OFS   rippath;        /* Default icon & RIP screens path           */

  /* More help files */

  OFS   hlp_hdrentry;   /* Help screen for message header entry      */
  OFS   hlp_msgentry;   /* Help screen for start of message entry    */
  OFS   not_configured; /* Displayed if user is not configured       */

  /* MCP stuff */

  OFS   mcp_pipe;       /* Default MCP pipe                          */

  /* Paths to the new marea/farea message/file data files */

  OFS   marea_name;     /* Name of message area data file            */
  OFS   farea_name;     /* Name of file area data file               */

  /* Caller information log */

  OFS   caller_log;     /* Caller information log                    */
};


/*---------------------------------------------------------------------------*/

/*
**  Structure of Max 3.0 MAREA.DAT
*/

#define MAREA_STRING(ah, var) (heap + (ah).var)

typedef struct _ovride
{
  /* Only one of 'opt' or 'name' should be used.  If opt==0, use name.  If  *
   * name==0, use opt.                                                      */

  WrdT   opt;           /* Type of menu option to override     ...OR...     */
  BytT   name;          /* First letter of command to override              */
  BytT   rsvd1;         /* Reserved for future use                          */
  WrdT   acs;           /* New ACS required to access option                */
  WrdT   menuname;      /* Use this access level on the given menu only     */
} OVERRIDE_MENU;


/* Bit masks for the ma.attribs field */

#define MA_PVT      0x0001  /* Private msgs allowed */
#define MA_PUB      0x0002  /* Public msgs allowed */
#define MA_HIBIT    0x0004  /* High bit msgs allowed */

#define MA_NET      0x0008  /* Netmail area */
#define MA_ECHO     0x0010  /* Echomail area */
#define MA_CONF     0x0020  /* Conference area */

#define MA_ANON     0x0040  /* Anonymous messages are OK */
#define MA_NORNK    0x0080  /* Don't use the REALNAME kludge for this area */
#define MA_REAL     0x0100  /* Force  use of       real name for this area */
#define MA_ALIAS    0x0200  /* Force  use of alias      name for this area */
#define MA_AUDIT    0x0400  /* Use auditing (msg tracking) controls in area*/
#define MA_READONLY 0x0800  /* Area is read-only */
#define MA_HIDDN    0x1000  /* Area does not display on normal area list   */
#define MA_ATTACH   0x2000  /* Area allows local file attaches             */
#define MA_DIVBEGIN 0x4000  /* A message area division, not a real area    */
#define MA_DIVEND   0x8000  /* End of the message area division            */

#define MA2_NOMCHK  0x0001  /* Don't do personal mail check in this area   */

#define MA_SHARED   (MA_ECHO | MA_CONF)

typedef struct _msgarea
{
  WrdT cbArea;          /* Length of THIS INDIVIDUAL RECORD                0*/
  WrdT num_override;    /* Number of overrides following this record       2*/
  WrdT cbHeap;          /* Length of the zstr heap following the overrides 4*/
  WrdT division;        /* Reserved for future use                         6*/
  WrdT name;            /* String format of area's name.                   8*/
  WrdT acs;             /* Access control string for this area            10*/
  WrdT path;            /* Path to messages (but for MA_DIVBEGIN only,    12*
                         * used instead as name of custom .bbs file).       */
  WrdT echo_tag;        /* The 'tag' of the area, for use in ECHOTOSS.LOG 14*/
  WrdT descript;        /* The DIR.BBS-like description for msg section   16*/
  WrdT origin;          /* The ORIGIN line for this area                  18*/
  WrdT menuname;        /* Custom menu name                               20*/
  WrdT menureplace;     /* Replace this menu name with menuname from above22*/

  WrdT attribs;         /* Attributes for this area                       24*/

  NETADDR primary;      /* Use as primary address for this area           26*/
  NETADDR seenby;       /* Use as address in seen-bys                     34*/
  WrdT attribs_2;       /* More attributes                                42*/
  WrdT type;            /* Message base type.  MSGTYPE_SDM = *.MSG.       44*
                         * MSGTYPE_SQUISH = SquishMail.  (Constants are     *
                         * in MSGAPI.H)                                     */
  WrdT killbyage;       /* Make sure msgs are less than X days old        46*/
  WrdT killbynum;       /* Make sure there are less than X msgs           48*/
  WrdT killskip;        /* Exempt the first X msgs from this processing   50*/
  WrdT barricade;       /* Barricade file                                 52*/
  WrdT barricademenu;   /* Apply barricade priv while using this menu     54*/
  LngT cbPrior;         /* Seek offset from start of this area to get back56*
                         * to prior area.                                   */
  WrdT attachpath;      /* Reserved for future use                        58*/
  DwdT rsvd4;                                                           /*60*/
}
MAREA;                                                                  /*64*/


/*---------------------------------------------------------------------------*/

/*
**  Structure definition for the old-style
**  Maximus AREAS.DAT file (version 2.x).
*/

typedef struct
{
    LngT id;              /* Unique identifier for AREA.DAT structure.       *
                           * Should be AREA_id, above.                       */

    WrdT struct_len;      /* Length of _area structure -- this needs only    *
                           * to be read from the first record in an area     *
                           * data file, since it can be assumed to remain    *
                           * the same throughout the entire file.  This is   *
                           * GUARANTEED to be at offset four for this and    *
                           * all future versions of this structure.          */

    WrdT areano;          /* OBSOLETE.  Two-byte integer representation of   *
                           * this area's name.  Use area.name instead.       */

    BytT name[40];        /* String format of area's name.  USE THIS!        */

    /*************************************************************************/
    /**                        Message Area Information                     **/
    /*************************************************************************/

    WrdT type;            /* Message base type.  MSGTYPE_SDM = *.MSG.        *
                           * MSGTYPE_SQUISH = SquishMail.  (Constants are    *
                           * in MSGAPI.H)                                    */

    BytT msgpath[80];     /* Path to messages                                */
    BytT msgname[40];     /* The 'tag' of the area, for use in ECHOTOSS.LOG  */
    BytT msginfo[80];     /* The DIR.BBS-like description for msg section    */
    BytT msgbar[80];      /* Barricade file for message area                 */
    BytT origin[62];      /* The ORIGIN line for this area                   */

    IntT msgpriv;         /* This is the priv required to access the msg     *
                           * section of this area.                           */
    BytT fill0;           /* The lock for the message area (obsolete)        */

    BytT fill1;

    IntT origin_aka;      /* This is the AKA number to use on the origin     *
                           * line.  See the normal SysOp documentation on    *
                           * the "Origin" statement, for info on how this    *
                           * number is used.                                 */

    /*************************************************************************/
    /**                        File Area Information                        **/
    /*************************************************************************/


    BytT filepath[80];    /* Path for downloads                              */
    BytT uppath[80];      /* Path for uploads                                */
    BytT filebar[80];     /* Barricade file for file areas                   */
    BytT filesbbs[80];    /* Path to FILES.BBS-like catalog for this area    */
    BytT fileinfo[80];    /* The DIR.BBS-like description for file section   */

    IntT filepriv;        /* This is the priv required to access the file    *
                           * section of this area.                           */
    BytT fill15;          /* The locks for the file area (obsolete)          */
    BytT fill2;

    /*************************************************************************/
    /**                      Miscellaneous Information                      **/
    /*************************************************************************/


    BytT msgmenuname[13]; /* Alternate *.MNU name to use for this msg.area   */
    BytT filemenuname[13];/* Alternate *.MNU name to use for this file area  */

    WrdT attrib[12];      /* This is an array of attributes for the          *
                           * msg/file areas.  These are dependant on PRIV    *
                           * level.  Once you have the CLASS number for a    *
                           * particular user (via Find_Class_Number()), you  *
                           * can find the attributes for that particular     *
                           * priv level like this: "area.attrib[class]"      *
                           * ...which will get you the attribute for that    *
                           * priv level.                                     */

    /*************************************************************************/
    /**                      Stuff hacked on later                          **/
    /*************************************************************************/

    BytT movr[8][16];     /* Message area override privs                     */
    BytT fovr[8][16];     /* File area override privs                        */

    DwdT msglock;         /* 32-bit locks for message areas                  */
    DwdT filelock;        /* 32-bit locks for file areas                     */

    WrdT killbyage;       /* MAXREN: max # of days to keep msgs in this area */
                          /*         (use 0 for no deletion by age)          */
    WrdT killbynum;       /* MAXREN: max # of msgs to keep in area (use 0    */
                          /*         for no deletion by #msgs.)              */

}
OLD_AREADAT;


